#include "hashedip"

int HashedIp::target(struct in_addr clientip) {
    // Hash the client's IP into an index
    unsigned h = 0;
    for (char *cp = (char*)&clientip;
	 unsigned(cp - (char*)&clientip) < sizeof(struct in_addr);
	 cp++) {
	h += *cp;
	h %= balancer.nbackends();
    }   
    int index = int(h);

    if (config.verbose()) {
	ostringstream o;
	o << index;
	msg ("Client IP " + (string)inet_ntoa(clientip) + " hashes to " +
	     o.str() + ", back end " +
	     balancer.backend(index).description() +
	     "\n");
    }

    // In strict mode, back end must be available, or don't proceed
    // In lax mode, fall back to least-connections dispatching
    if (! balancer.backend(index).available()) {
	if (config.dispatchmode() == Dispatchmode::m_strict_hashed_ip)
	    throw static_cast<Error>("Hashed-IP algorithm: target back end ") +
		   balancer.backend(index).description() + " unavailable";
	else {
	    msg ("Hashed-IP algorithm: target back end " +
		 balancer.backend(index).description() + " unavailable, "
		 "falling back to least-connections\n");
	    Leastconn l;
	    index = l.target(clientip);
	}
    }

    // Got it
    return (index);
}
